

/*
 * Boot support
 */
#include <common.h>
#include <command.h>
#include <net.h>
//--> porting from VT8500 UBOOT for uploading file to tftp server (howyahuo on 20091214)
#include "../net/tftp.h"
#include "../board/wmt/flash/spi_flash.h"

#define UBOOT_TFTPILE_ARG_MAX	5
//<-- end porting

#if (CONFIG_COMMANDS & CFG_CMD_NET)


extern int do_bootm (cmd_tbl_t *, int, int, char *[]);

static int netboot_common (proto_t, cmd_tbl_t *, int , char *[]);

int do_bootp (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	return netboot_common (BOOTP, cmdtp, argc, argv);
}

U_BOOT_CMD(
	bootp,	3,	1,	do_bootp,
	"bootp\t- boot image via network using BootP/TFTP protocol\n",
	"[loadAddress] [bootfilename]\n"
);

int do_tftpb (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
//--> porting from VT8500 UBOOT for uploading file to tftp server (howyahuo on 20091214)
	struct tftp_agrument_tag tftparg;

	tftparg.optdirect = 1; // get file from server
	set_tftparg(&tftparg);
//<-- end porting
	return netboot_common (TFTP, cmdtp, argc, argv);
}

U_BOOT_CMD(
	tftpboot,	3,	1,	do_tftpb,
	"tftpboot- boot image via network using TFTP protocol\n",
	"[loadAddress] [bootfilename]\n"
);

int do_rarpb (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	return netboot_common (RARP, cmdtp, argc, argv);
}

U_BOOT_CMD(
	rarpboot,	3,	1,	do_rarpb,
	"rarpboot- boot image via network using RARP/TFTP protocol\n",
	"[loadAddress] [bootfilename]\n"
);

#if (CONFIG_COMMANDS & CFG_CMD_DHCP)
int do_dhcp (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	return netboot_common(DHCP, cmdtp, argc, argv);
}

U_BOOT_CMD(
	dhcp,	3,	1,	do_dhcp,
	"dhcp\t- invoke DHCP client to obtain IP/boot params\n",
	"\n"
);
#endif	/* CFG_CMD_DHCP */

#if (CONFIG_COMMANDS & CFG_CMD_NFS)
int do_nfs (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	return netboot_common(NFS, cmdtp, argc, argv);
}

U_BOOT_CMD(
	nfs,	3,	1,	do_nfs,
	"nfs\t- boot image via network using NFS protocol\n",
	"[loadAddress] [host ip addr:bootfilename]\n"
);
#endif	/* CFG_CMD_NFS */

static void netboot_update_env (void)
{
	char tmp[22];

	if (NetOurGatewayIP) {
		ip_to_string (NetOurGatewayIP, tmp);
		setenv ("gatewayip", tmp);
	}

	if (NetOurSubnetMask) {
		ip_to_string (NetOurSubnetMask, tmp);
		setenv ("netmask", tmp);
	}

	if (NetOurHostName[0])
		setenv ("hostname", NetOurHostName);

	if (NetOurRootPath[0])
		setenv ("rootpath", NetOurRootPath);

	if (NetOurIP) {
		ip_to_string (NetOurIP, tmp);
		setenv ("ipaddr", tmp);
	}

	if (NetServerIP) {
		ip_to_string (NetServerIP, tmp);
		setenv ("serverip", tmp);
	}

	if (NetOurDNSIP) {
		ip_to_string (NetOurDNSIP, tmp);
		setenv ("dnsip", tmp);
	}
#if (CONFIG_BOOTP_MASK & CONFIG_BOOTP_DNS2)
	if (NetOurDNS2IP) {
		ip_to_string (NetOurDNS2IP, tmp);
		setenv ("dnsip2", tmp);
	}
#endif
	if (NetOurNISDomain[0])
		setenv ("domain", NetOurNISDomain);

#if (CONFIG_COMMANDS & CFG_CMD_SNTP) && (CONFIG_BOOTP_MASK & CONFIG_BOOTP_TIMEOFFSET)
	if (NetTimeOffset) {
		sprintf (tmp, "%d", NetTimeOffset);
		setenv ("timeoffset", tmp);
	}
#endif
#if (CONFIG_COMMANDS & CFG_CMD_SNTP) && (CONFIG_BOOTP_MASK & CONFIG_BOOTP_NTPSERVER)
	if (NetNtpServerIP) {
		ip_to_string (NetNtpServerIP, tmp);
		setenv ("ntpserverip", tmp);
	}
#endif
}

static int
netboot_common (proto_t proto, cmd_tbl_t *cmdtp, int argc, char *argv[])
{
	char *s;
	int   rcode = 0;
	int   size;

	/* pre-set load_addr */
	if ((s = getenv("loadaddr")) != NULL) {
		load_addr = simple_strtoul(s, NULL, 16);
	}

	switch (argc) {
	case 1:
		break;

	case 2:	/* only one arg - accept two forms:
		 * just load address, or just boot file name.
		 * The latter form must be written "filename" here.
		 */
		if (argv[1][0] == '"') {	/* just boot filename */
			copy_filename (BootFile, argv[1], sizeof(BootFile));
		} else {			/* load address	*/
			load_addr = simple_strtoul(argv[1], NULL, 16);
		}
		break;

	case 3:	load_addr = simple_strtoul(argv[1], NULL, 16);
		copy_filename (BootFile, argv[2], sizeof(BootFile));

		break;
//--> porting from VT8500 UBOOT for uploading file to tftp server (howyahuo on 20091214)
	case UBOOT_TFTPILE_ARG_MAX: // for tftpf
		break;
//<-- end porting

	default: printf ("Usage:\n%s\n", cmdtp->usage);
		return 1;
	}

	if ((size = NetLoop(proto)) < 0)
		return 1;

	/* NetLoop ok, update environment */
	netboot_update_env();

	/* done if no file was loaded (no errors though) */
	if (size == 0)
		return 0;

	/* flush cache */
	flush_cache(load_addr, size);

	/* Loading ok, check if we should attempt an auto-start */
	if (((s = getenv("autostart")) != NULL) && (strcmp(s,"yes") == 0)) {
		char *local_args[2];
		local_args[0] = argv[0];
		local_args[1] = NULL;

		printf ("Automatic boot of image at addr 0x%08lX ...\n",
			load_addr);
		rcode = do_bootm (cmdtp, 0, 1, local_args);
	}

#ifdef CONFIG_AUTOSCRIPT
	if (((s = getenv("autoscript")) != NULL) && (strcmp(s,"yes") == 0)) {
		printf("Running autoscript at addr 0x%08lX ...\n", load_addr);
		rcode = autoscript (load_addr);
	}
#endif
	return rcode;
}

#if (CONFIG_COMMANDS & CFG_CMD_PING)
int do_ping (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	if (argc < 2)
		return -1;

	NetPingIP = string_to_ip(argv[1]);
	if (NetPingIP == 0) {
		printf ("Usage:\n%s\n", cmdtp->usage);
		return -1;
	}

	if (NetLoop(PING) < 0) {
		printf("ping failed; host %s is not alive\n", argv[1]);
		return 1;
	}

	printf("host %s is alive\n", argv[1]);

	return 0;
}

U_BOOT_CMD(
	ping,	2,	1,	do_ping,
	"ping\t- send ICMP ECHO_REQUEST to network host\n",
	"pingAddress\n"
);
#endif	/* CFG_CMD_PING */

#if (CONFIG_COMMANDS & CFG_CMD_CDP)

static void cdp_update_env(void)
{
	char tmp[16];

	if (CDPApplianceVLAN != htons(-1)) {
		printf("CDP offered appliance VLAN %d\n", ntohs(CDPApplianceVLAN));
		VLAN_to_string(CDPApplianceVLAN, tmp);
		setenv("vlan", tmp);
		NetOurVLAN = CDPApplianceVLAN;
	}

	if (CDPNativeVLAN != htons(-1)) {
		printf("CDP offered native VLAN %d\n", ntohs(CDPNativeVLAN));
		VLAN_to_string(CDPNativeVLAN, tmp);
		setenv("nvlan", tmp);
		NetOurNativeVLAN = CDPNativeVLAN;
	}

}

int do_cdp (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	int r;

	r = NetLoop(CDP);
	if (r < 0) {
		printf("cdp failed; perhaps not a CISCO switch?\n");
		return 1;
	}

	cdp_update_env();

	return 0;
}

U_BOOT_CMD(
	cdp,	1,	1,	do_cdp,
	"cdp\t- Perform CDP network configuration\n",
);
#endif	/* CFG_CMD_CDP */

#if (CONFIG_COMMANDS & CFG_CMD_SNTP)
int do_sntp (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	char *toff;

	if (argc < 2) {
		NetNtpServerIP = getenv_IPaddr ("ntpserverip");
		if (NetNtpServerIP == 0) {
			printf ("ntpserverip not set\n");
			return (1);
		}
	} else {
		NetNtpServerIP = string_to_ip(argv[1]);
		if (NetNtpServerIP == 0) {
			printf ("Bad NTP server IP address\n");
			return (1);
		}
	}

	toff = getenv ("timeoffset");
	if (toff == NULL) NetTimeOffset = 0;
	else NetTimeOffset = simple_strtol (toff, NULL, 10);

	if (NetLoop(SNTP) < 0) {
		printf("SNTP failed: host %s not responding\n", argv[1]);
		return 1;
	}

	return 0;
}

U_BOOT_CMD(
	sntp,	2,	1,	do_sntp,
	"sntp\t- synchronize RTC via network\n",
	"[NTP server IP]\n"
);
#endif	/* CFG_CMD_SNTP */

//--> porting from VT8500 UBOOT for uploading file to tftp server (howyahuo on 20091214)
//Author: Hang Yan

/*
 * Brief:
 *	Check whether one string is digital string.
 * Input:
 *	str -- the checked string;
 *	base -- 10: decimal; 16: hexadecimal;
 * Return:
 *	>=0 -- the string is digital string;
 *	-1 -- the string isn't digital string;
 * History:
 *	Created	2009-2-12
 * Author:
 * 	Hang Yan in ShenZhen.
 */
int is_digitalstr(char* str, int base)
{
//	int ret = 0;
	char* p = str;

	if ((NULL == str) || (*str == '\0') ||
		((base != 10) && (base != 16)))
	{
		return -1;
	}
	//
	if (base == 16)
	{
		if ((str[1] != '\0') &&
			(str[0] == '0') &&
			((str[1] == 'x') || (str[1] == 'X')))
		{
			if (str[2] != '\0')
			{
				p = str + 2;
			} else
			{
				return -1;
			}
		}
	}
	do
	{
		switch (base)
		{
			case 10:
				if ((*p < '0') || (*p > '9'))
				{
					return -1;
				}
				break;
			case 16:
				if (((*p < 'a') || (*p > 'z')) &&
					((*p < 'A') || (*p > 'Z')) &&
					((*p < '0') || (*p > '9')))
				{
					return -1;
				}
				break;
			default: // never reach it
				break;
		};
		p++;
	} while(*p != '\0');
	return 0;
}

/*
 * Brief:
 *	Check the filename is valid.
 * Input:
 *	filename -- one string of one file.
 * Return:
 *	>=0 -- the input is valid file name(digital,letter,'_','.');
 *	< 0 -- the input is invalid file name;
 * History:
 *	Created	2009-2-12
 * Author:
 * 	Hang Yan in ShenZhen.
 */
int is_validfilename(char* filename)
{
	char* p = filename;

	if ((NULL == filename) || (*filename == '\0'))
	{
		return -1;
	}
	do {
		if ((*p != '_') && (*p != '.') &&
			((*p < '0') || (*p > '9')) &&
			((*p < 'a') || (*p > 'z')) &&
			((*p < 'A') || (*p > 'Z')))
		{
			return -1;
		}
		p++;
	} while (*p != '\0');
	return 0;
}
/*
 * Brief:
 *	Transfer file by TFTP. Now only support putting file to server.
 * History:
 *	Created	2009-2-12
 * Author:
 * 	Hang Yan in ShenZhen.
 */
int do_tftpf (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	int ret = 0;
	struct tftp_agrument_tag tftparg;

	// check the format of argument for command line
	if (argc < UBOOT_TFTPILE_ARG_MAX)
	{
		ret = -1;
		goto out;
	}
	switch (argv[1][1])
	{
		case 'p':
			tftparg.optdirect = 2; // put file to server
			break;
		default:
			ret = -1;
			goto out;
			break;
	};
	if ((is_digitalstr(argv[3],16) < 0) || (is_digitalstr(argv[4],16) < 0))
	{
		ret = -1;
		goto out;
	}
	if (is_validfilename(argv[2]) < 0)
	{
		printf("Input file name is invalid.\n");
		goto out;
	}
	// set tftp argument
	strncpy(tftparg.filename, argv[2], sizeof(tftparg.filename)-1); // file name
	tftparg.spistartaddr = simple_strtoul(argv[3], NULL, 16);
//--> removed by howayhuo on 20100603
/*
	if (tftparg.spistartaddr < SF_START_ADDR)
	{
		printf("SPI Flash start addr shouldn't be little than 0x%08x.\n", SF_START_ADDR);
		goto out;
	}
*/
//<--
	tftparg.offset = simple_strtoul(argv[4], NULL, 16);
	if (tftparg.offset < 1)
	{
		printf("read length should >=1.\n");
		goto out;
	}
	if ((tftparg.spistartaddr+tftparg.offset) <= tftparg.spistartaddr)
	{
		// the address will wrap
		tftparg.offset = (unsigned long)0xFFFFFFFF - tftparg.spistartaddr +1;
	}
	set_tftparg(&tftparg);
	// tftp operation
	return netboot_common (TFTP, cmdtp, argc, argv);
out:
	printf("tftpfile command format error.\n");
	return ret;
}

U_BOOT_CMD(
	uploadfile, UBOOT_TFTPILE_ARG_MAX, 1,	do_tftpf,
	"uploadfile- Transfer the spi flash image to the server.\n",
	"uploadfile [-p] [filename] [StartAddress] [offset]\n"
	"    -p   put file.\n"
	"    filename: the saved file name.\n"
	"    StartAddress: the saved start address of spi flash.\n"
	"    offset: the saved length.\n"
	"Note: \n"
	"     StartAddress and offset are both hexademical.\n"
	"Example:\n"
	"     uploadfile -p spi.bin fff80000 80000\n"
	"  TFTP sever will get the file spi.bin which consists of the spi flash arrange \n"
	"from 0xfff8000 to 0xffffffff."
);

//<-- end porting

#endif	/* CFG_CMD_NET */
