
 
#include <common.h>
#include <command.h>
#include <malloc.h>
#include <asm/io.h>
#include <watchdog.h>
/* #include <env.h> */
#include <image.h>
#include <linux/stddef.h>
#include <asm/byteorder.h>

//#define DEBUG_CMD_RANDOMMAC

int do_randommac (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[]);

U_BOOT_CMD(
	randommac,	CFG_MAXARGS,	1,	do_randommac,
	"randmac - generate a random MAC address and save to \"ethaddr\" environment variable\n",
	"no parameters\n"
);

/*
 * Name:
 * 	generate_randommac(...)
 * Brief:
 *	Generate one random mac address.
 * History:
 *	created		2008-11-20
 * Author:
 *	Hang Yan in ShenZhen.
 */
#define RANDOMMAC_OUR_MAC_OUI_BYTE	0x004063
#define RANDOMMAC_OUR_MAC_OUI_LEN	3

void generate_randommac(unsigned char mac[6])
{
	int i;
	unsigned long random;

	/* Use the assigned OUI */
	for (i = 0; i <= (RANDOMMAC_OUR_MAC_OUI_LEN-1); i++)
	{
		mac[i] = (RANDOMMAC_OUR_MAC_OUI_BYTE>>((RANDOMMAC_OUR_MAC_OUI_LEN-1-i)*8))&0x0FF;
	};
	/* Generate other bytes in mac address */
	for (i = RANDOMMAC_OUR_MAC_OUI_LEN; i < 6; i++)
	{
		random = (get_timer(0)%3072)*1664525L+1013904223L;
		mac[i] = random & 0x0FF;
	};
	#ifdef DEBUG_CMD_RANDOMMAC
	printf("generate_randommac: ");
	for (i = 0; i < 6; i++)
	{
		printf("%02x:", mac[i]);
	};
	printf("\n");
	#endif

	return;	
}

/*
 * Name:
 * 	mac_to_string(...)
 * Brief:
 *	Convert mac array to one string.
 *	eg.  converting 00406357E7C3 to 00:40:63:57:E7:C3  
 * History:
 *	created		2008-11-20
 * Author:
 *	Hang Yan in ShenZhen.
 */
int mac_to_string(unsigned char mac[6], char strmac[18])
{	
	unsigned short i;
	unsigned short j;
	unsigned char bd;

	#ifdef DEBUG_CMD_RANDOMMAC
	printf("mac_to_string() before input mac: ");
	for (i = 0; i < 6; i++)
	{
		printf("%02x:", mac[i]);
	};
	printf("\n");
	#endif
	for (i = 0; i < 6; i++)
	{
		for (j = 0; j <= 1; j++)
		{
			bd = (mac[i]>>((1-j)*4))&0x0F;
			if (bd <= 9)
			{
				strmac[3*i+j] = bd + 48; /* '0' - '9'*/
			} else {
				strmac[3*i+j] = bd - 10 + 65; /* 'A' - 'F' */
			}
		};
		if (i < 5)
		{
			strmac[3*i+j] = ':';
		} else {
			strmac[3*i+j] = '\0';
		}
	};
	#ifdef DEBUG_CMD_RANDOMMAC
	printf("after mac_to_string input mac: ");
	for (i = 0; i < 6; i++)
	{
		printf("%02x:", mac[i]);
	};
	printf("\n output macstr:%s\n", strmac);
	#endif
	return 0;
}

int
do_randommac (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	char mac[6];
	char tembuf[32];
	int i;
	
  /* Generate on random mac address */
	generate_randommac(mac);
	#ifdef DEBUG_CMD_RANDOMMAC
	printf("after generate ->mac:");
	for (i = 0; i < 6; i++)
	{
		printf("%02x:", mac[i]);
	};
	#endif
	mac_to_string(mac, tembuf);
	setenv("ethaddr", tembuf); //Save the mac
	//#ifdef DEBUG_CMD_RANDOMMAC
	printf("The random mac: %s\n", tembuf);
	//#endif	
	
  return 0;
}
