

/*
 * serial.c - serial support for the WMT evaluation board
 */

#include <common.h>

#ifdef CFG_WMT_SERIAL

#include <asm/arch/hardware.h>
#include "uart.h"
#include "gpio.h"

DECLARE_GLOBAL_DATA_PTR;

PUART_REG	pUart_Reg = (PUART_REG)BA_UART0;
PGPIO_REG	pGpioReg = (PGPIO_REG)BA_GPIO;

/*
 * Function purpose:
 *				UART initialization
 */
int serial_init(void)
{
	/* 2006-12-06 WayneLee@via.com.tw */
	/* Switch the Uart's pin from default GPIO into uart function pins for UART0/UART1 */
	pGpioReg->CTRL_UR_LPC_SF &= ~(GPIO_UR0_ALL | GPIO_UR1_ALL);

	serial_setbrg();

	return 0;
}

/*
 * Function purpose:
 *				Output character data through UART
 */
void
serial_putc(const char c)
{
	while (1) {
		if ((pUart_Reg->URUSR & URUSR_TXDBSY) == 0) 	/* TX data not busy */
		break;
	}

	pUart_Reg->URTDR = c;
	if (c == '\n')
		serial_putc('\r');
}

/*
 * Function purpose:
 *				Output string data through UART
 */
void
serial_puts(const char *s)
{
	while (*s)
		serial_putc(*s++);
}

/*
 * Function purpose:
 *				Require character data from UART
 */
int
serial_getc(void)
{
	while (!(pUart_Reg->URUSR & URUSR_RXDRDY))
		;

	return pUart_Reg->URRDR&0xff;
}

/*
 * Function purpose:
 *				return RXDRDY status bit in Uart status register
 */
int
serial_tstc(void)
{
	return (pUart_Reg->URUSR & URUSR_RXDRDY) ? 1 : 0;
}

/*
 * Function purpose:
 *				Uart baud rate setting and initialization
 * Note:
 *				Relative settings according to UART/IrDA
 *				Programmer Guide Revision 0.13
 */
void
serial_setbrg(void)
{
	unsigned long TmpDiv = 0;
	unsigned long TmpBkr = 0;

	switch (gd->baudrate) {
	/*
	case BR_921K6:
		TmpDiv = UART_BR_921K6;
		TmpBkr = UART_BK_921K6;
		break;

	case BR_460K8:
		TmpDiv = UART_BR_460K8;
		TmpBkr = UART_BK_460K8;
		break;

	case BR_230K4:
		TmpDiv = UART_BR_230K4;
		TmpBkr = UART_BK_230K4;
		break;
	*/

	case BR_57K6:
		TmpDiv = UART_BR_57K6;
		TmpBkr = UART_BK_57K6;
		break;
	/*
	case BR_28K8:
		TmpDiv = UART_BR_28K8;
		TmpBkr = UART_BK_28K8;
		break;

	case BR_14K4:
		TmpDiv = UART_BR_14K4;
		TmpBkr = UART_BK_14K4;
		break;

	case BR_7K2:
		TmpDiv = UART_BR_7K2;
		TmpBkr = UART_BK_7K2;
		break;

	case BR_3K6:
		TmpDiv = UART_BR_3K6;
		TmpBkr = UART_BK_3K6;
		break;
	*/

	case BR_38K4:
		TmpDiv = UART_BR_38K4;
		TmpBkr = UART_BK_38K4;
		break;

	case BR_19K2:
		TmpDiv = UART_BR_19K2;
		TmpBkr = UART_BK_19K2;
		break;

	case BR_9K6:
		TmpDiv = UART_BR_9K6;
		TmpBkr = UART_BK_9K6;
		break;

	case BR_115K2:
	default:
		TmpDiv = UART_BR_115K2;
		TmpBkr = UART_BK_115K2;
		break;
	}

	pUart_Reg->URDIV = TmpDiv;
	pUart_Reg->URBKR = TmpBkr;

	/* Disable TX,RX */
	pUart_Reg->URLCR = 0;

	/* Disable all interrupt */
	pUart_Reg->URIER = 0;

	/* 2006-12-06 WayneLee@via.com.tw */
	/* Clear all interrupt status until all zeros */
	/*
	while (pUart_Reg->URISR) {
		pUart_Reg->URISR = pUart_Reg->URISR;
	}
	*/

	/* Reset Tx,Rx Fifo until all zeros */
	while (pUart_Reg->URFCR) {
		/* Reset TX,RX Fifo */
		pUart_Reg->URFCR = URFCR_TXFRST | URFCR_RXFRST;
	}

	/* Disable Fifo */
	pUart_Reg->URFCR &= (~URFCR_FIFOEN);

	/* 8-N-1 */
	pUart_Reg->URLCR |= (URLCR_DLEN8B & ~URLCR_STBLEN2b & ~URLCR_PYTEN);

	/* Enable TX,RX */
	pUart_Reg->URLCR |= URLCR_RXEN | URLCR_TXEN;
}

#endif	/* CFG_WMT_SERIAL */
