/*
 * Copyright (c) 2008 WonderMedia Technologies, Inc. All Rights Reserved.
 *
 * This PROPRIETARY SOFTWARE is the property of WonderMedia Technologies, Inc.
 * and may contain trade secrets and/or other confidential information of
 * WonderMedia Technologies, Inc. This file shall not be disclosed to any third
 * party, in whole or in part, without prior written consent of WonderMedia.
 *
 * THIS PROPRIETARY SOFTWARE AND ANY RELATED DOCUMENTATION ARE PROVIDED AS IS,
 * WITH ALL FAULTS, AND WITHOUT WARRANTY OF ANY KIND EITHER EXPRESS OR IMPLIED,
 * AND WonderMedia TECHNOLOGIES, INC. DISCLAIMS ALL EXPRESS OR IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR
 * NON-INFRINGEMENT.
 */

#ifndef __NAND_H__
#define __NAND_H__

#define __NFC_BASE 0xd8009000
#define __SCC_BASE 0xd8120000

typedef struct WMT_NFC_CFG_t{
	volatile unsigned int NFCR0;           /* 0x00 */
	volatile unsigned int	NFCR1;           /* 0x04 */
	volatile unsigned int	NFCR2;           /* 0x08 */
	volatile unsigned int	NFCR3;           /* 0x0c */
	volatile unsigned int	NFCR4;           /* 0x10 */
	volatile unsigned int NFCR5;           /* 0x14 */
	volatile unsigned int	NFCR6;           /* 0x18 */
	volatile unsigned int	NFCR7;           /* 0x1c */
	volatile unsigned int	NFCR8;           /* 0x20 */
	volatile unsigned int	NFCR9;           /* 0x24 */
	volatile unsigned int NFCRa;           /* 0x28 */
	volatile unsigned int	NFCRb;           /* 0x2c */
	volatile unsigned int	NFCRc;           /* 0x30 */
	volatile unsigned int	NFCRd;           /* 0x34 */
	volatile unsigned int NFCRe;           /* 0x38 */
	volatile unsigned int NFCRf;           /* 0x3c */
	volatile unsigned int NFCR10;          /* 0x40 */
	volatile unsigned int NFCR11;          /* 0x44 */
	volatile unsigned int NFCR12;          /* 0x48 */
	volatile unsigned int NFCR13;          /* 0x4c */
	volatile unsigned int NFCR14;          /* 0x50 */
	volatile unsigned int NFCR15;          /* 0x54 */
	volatile unsigned int NFCR16;          /* 0x58 */
	volatile unsigned int NFCR17;          /* 0x5c */
	volatile unsigned int NFCR18;          /* 0x60 */
	volatile unsigned int NFCR19;          /* 0x64 */
	volatile unsigned int NFCR1a;          /* 0x68 */
	volatile unsigned int NFCR1b;          /* 0x6c */
	volatile unsigned int NFCR1c;          /* 0x70 */
	volatile unsigned int NFCR1d;          /* 0x74 */
	volatile unsigned int NFCR1e;          /* 0x78 */
	volatile unsigned int NFCR1f;          /* 0x7c */
	volatile unsigned int NFCR20;          /* 0x80 */
	volatile unsigned int NFCR21;          /* 0x84 */
	volatile unsigned int NFCR22;          /* 0x88 */
	volatile unsigned int NFCR23;          /* 0x8c */
	volatile unsigned int NFCR24;          /* 0x90 */
	volatile unsigned int NFCR25;          /* 0x94 */
	volatile unsigned int NFCR26;          /* 0x98 */
	volatile unsigned int NFCR27;          /* 0x9c */
	volatile unsigned int NFCR28;          /* 0xa0 */
	volatile unsigned int NFCR29;          /* 0xa4 */
	volatile unsigned int NFCR2a;          /* 0xa8 */
	volatile unsigned int NFCR2b;          /* 0xac */
	volatile unsigned int NFCR2c;          /* 0xb0 */
	volatile unsigned int NFCR2d;          /* 0xb4 */
	volatile unsigned int NFCR2e;          /* 0xb8 */
	volatile unsigned int RESERVEDe[0x01]; /* 0xbc */
	volatile unsigned char FIFO[64];       /* 0xc0 ~ 0xfc */
	volatile unsigned int	DMA_SAR;
	volatile unsigned int	RESERVE;
	volatile unsigned int	DMA_TCR;
	volatile unsigned int	DMA_CCR;
	volatile unsigned int	RESERVED1C[0x1c];
	volatile unsigned int	DMA_CTR;
	volatile unsigned int	RESERVED3[0x03];
	volatile unsigned int	DMA_GCR;
	volatile unsigned int	DMA_GSR;
} WMT_NFC_CFG, *WMT_PNFC_CFG;

typedef struct wmtnfc_setting_s
{
	unsigned int	nfc12;
	unsigned int	nfc14;
	unsigned int	nfc17;
	unsigned int 	divisor;
}wmtnfc_setting_t;

typedef struct WMTNF_INFO_s
{
	unsigned int	id;
	unsigned int	oobblock;
	unsigned int	oobsize;
	unsigned int	blksize;
	unsigned int	page_shift;
	unsigned int	totblk;
	unsigned int	erasesize;
	unsigned int	col;
	unsigned int	row;
	unsigned int	page[2];
	unsigned int	ce;
	wmtnfc_setting_t wmt_nfc;
	unsigned int	ecc;
	unsigned int	whereisbbt[2];
}WMTNF_INFO_t;

#define MAX_PRODUCT_NAME_LENGTH     32
#define NAND_TYPE_SLC               0
#define NAND_TYPE_MLC               1
#define ECC_TYPE_HAMMING            0
#define ECC_TYPE_BCH                1

struct NAND_PRODUCT_INFO {
	unsigned int dwFlashID;       	/*composed by the first byte and the second byte*/
	unsigned int dwBlockCount;      /*how many blocks one chip have*/
	unsigned int dwPageSize;       /*page size*/

	unsigned int dwSpareSize;       /*spare area size*/
	unsigned int dwBlockSize;       /*block size*/
	unsigned int dwAddressCycle;      /*address cycle 4 or 5*/
	unsigned int dwBI0Position;      /*BI0 page postion in block*/
	unsigned int dwBI1Position;      /*BI1 page postion in block*/
	unsigned int dwBIOffset;       /*BI offset in page*/
	unsigned int dwDataWidth;      /*data with X8 or X16*/
	unsigned int dwPageProgramLimit;     /*chip can program PAGE_PROGRAM_LIMIT times within the same page*/
	unsigned int dwSeqRowReadSupport;    /*whether support sequential row read, 1 = support 0 = not support*/
	unsigned int dwSeqPageProgram;     /*chip need sequential page program in a block. 1 = need*/
	unsigned int dwNandType;       /*MLC or SLC*/
	unsigned int dwRWTimming;
 	char cProductName[MAX_PRODUCT_NAME_LENGTH]; /*product name. for example "HYNIX_NF_HY27UF081G2A"*/
};

/*
 *	NAND PDMA
 */
#define NAND_PDMA_BASE_ADDR 0xD8009100
#define NAND_DESC_BASE_ADDR 0x00D00000
struct _NAND_PDMA_REG_ {
	volatile unsigned long DMA_GCR;		/*	Rx00	*/
	volatile unsigned long DMA_IER;		/*	Rx04	*/
	volatile unsigned long DMA_ISR;		/*	Rx08	*/
	volatile unsigned long DMA_DESPR;	/*	Rx0C	*/
	volatile unsigned long DMA_RBR;		/*	Rx10	*/
	volatile unsigned long DMA_DAR;		/*	Rx14	*/
	volatile unsigned long DMA_BAR;		/*	Rx18	*/
	volatile unsigned long DMA_CPR;		/*	Rx1C	*/
	volatile unsigned long DMA_CCR;		/*	RX20	*/
	volatile unsigned long resv[5];  	/*	RX2C-3C	*/
} NAND_PDMA_REG, *PNAND_PDMA_REG;

/*
 *	NAND PDMA - DMA_GCR : DMA Global Control Register
 */
#define NAND_PDMA_GCR_DMA_EN			0x00000001      /* [0] -- DMA controller enable */
#define NAND_PDMA_GCR_SOFTRESET		0x00000100      /* [8] -- Software rest */

/*
 *	NAND PDMA - DMA_IER : DMA Interrupt Enable Register
 */
#define NAND_PDMA_IER_INT_EN			0x00000001      /* [0] -- DMA interrupt enable */
/*
 *	NAND PDMA - DMA_ISR : DMA Interrupt Status Register
 */
#define NAND_PDMA_IER_INT_STS			0x00000001      /* [0] -- DMA interrupt status */
/*
 *	NAND PDMA - DMA_DESPR : DMA Descriptor base address Pointer Register
 */

/*
 *	NAND PDMA - DMA_RBR : DMA Residual Bytes Register
 */
#define NAND_PDMA_RBR_End				0x80000000      /* [31] -- DMA descriptor end flag */
#define NAND_PDMA_RBR_Format			0x40000000      /* [30] -- DMA descriptor format */
/*
 *	NAND PDMA - DMA_DAR : DMA Data Address Register
 */

/*
 *	NAND PDMA - DMA_BAR : DMA Rbanch Address Register
 */

/*
 *	NAND PDMA - DMA_CPR : DMA Command Pointer Register
 */

/*
 *	NAND PDMA - DMA_CCR : DMAContext Control Register for Channel 0
 */
#define NAND_PDMA_READ					0x00
#define NAND_PDMA_WRITE					0x01
#define NAND_PDMA_CCR_RUN					0x00000080
#define NAND_PDMA_CCR_IF_to_peripheral	0x00000000
#define NAND_PDMA_CCR_peripheral_to_IF	0x00400000
#define NAND_PDMA_CCR_EvtCode				0x0000000f
#define NAND_PDMA_CCR_Evt_no_status		0x00000000
#define NAND_PDMA_CCR_Evt_ff_underrun		0x00000001
#define NAND_PDMA_CCR_Evt_ff_overrun		0x00000002
#define NAND_PDMA_CCR_Evt_desp_read		0x00000003
#define NAND_PDMA_CCR_Evt_data_rw			0x00000004
#define NAND_PDMA_CCR_Evt_early_end		0x00000005
#define NAND_PDMA_CCR_Evt_success			0x0000000f

/*
 *	PDMA Descriptor short
 */
struct _NAND_PDMA_DESC_S{
	unsigned long volatile ReqCount:16;		/* bit 0 -15 -Request count             */
	unsigned long volatile i:1;				/* bit 16    -interrupt                 */
	unsigned long volatile reserve:13;		/* bit 17-29 -reserved                  */
	unsigned long volatile format:1;		/* bit 30    -The descriptor format     */
	unsigned long volatile end:1;			/* bit 31    -End flag of descriptor list*/
	unsigned long volatile DataBufferAddr:32;/* bit 31    -Data Buffer address       */
} NAND_PDMA_DESC_S, *PNAND_PDMA_DESC_S;

/*
 *	PDMA Descriptor long
 */
struct _NAND_PDMA_DESC_L{
	unsigned long volatile ReqCount:16;		/* bit 0 -15 -Request count             */
	unsigned long volatile i:1;				/* bit 16    -interrupt                 */
	unsigned long volatile reserve:13;		/* bit 17-29 -reserved                  */
	unsigned long volatile format:1;		/* bit 30    -The descriptor format     */
	unsigned long volatile end:1;			/* bit 31    -End flag of descriptor list*/
	unsigned long volatile DataBufferAddr:32;/* bit 31-0  -Data Buffer address       */
	unsigned long volatile BranchAddr:32;	/* bit 31-2  -Descriptor Branch address	*/
	unsigned long volatile reserve0:32;		/* bit 31-0  -reserved */
} NAND_PDMA_DESC_L, *PNAND_PDMA_DESC_L;


#define DMA_SINGNAL 0
#define DMA_INC4 0x10
#define DMA_INC8 0x20

//#define NAND_LARGE_DWRESERVED1_OFS 4
#define NAND_LARGE_DWRESERVED1_OFS 9
#define NAND_SMALL_DWRESERVED1_OFS 0
//#define NAND_LARGE_DWRESERVED2_OFS 0
#define NAND_LARGE_DWRESERVED2_OFS 1
#define NAND_SMALL_DWRESERVED2_OFS 14

#define ADDR_COLUMN 1
#define ADDR_PAGE 2
#define ADDR_COLUMN_PAGE 3

/* err define */
#define ERR_NFC_CMD  1
#define ERR_NAND_IDLE  2
#define ERR_DMA_CFG 3
#define ERR_NAND_READY  4
#define ERR_NFC_READY  5
#define ERR_NO_BBT	   6
#define ERR_NAND_UNKNOW 7
#define ERR_NOT_SUPPORT 8
#define ERR_ECC_UNCORRECT 9
#define ERR_ECC_BANK	10
#define ERR_UNKNOW 11
#define ERR_NAND_BOUNDLY 12
#define ERR_BBT_BAD 13

#define ERR_IMAGE_FORMAT 13
/* nand id */
#define NAND_HYNIX 0xad
#define NAND_SAMSUNG 0xec
#define NAND_UNKONW 0x00

#define ECC_1bit_ERR 0x02

/*
 * Standard NAND flash commands
 */
#define NAND_READ0		0
#define NAND_READ1		1
#define NAND_READ2		0x50
#define NAND_PAGEPROG	0x10
#define NAND_ERASE_SET	0x60
#define NAND_STATUS		0x70
#define NAND_MULTI_STATUS	0x71
#define NAND_SEQIN		0x80
#define NAND_READID		0x90
#define NAND_ERASE_CONFIRM		0xd0
#define NAND_RESET		0xff

#define	NAND_READ_CONFIRM 0x30
#define	NAND_DUMMY_READ 0x03
#define NAND_DUMMY_PAGEPROG 0x11
#define NAND_CMD_COPYBACK 0x8A

/* Standard NAND Status bits */
#define NAND_STATUS_FAIL	0x01
#define PLANE0_FAIL				0x02
#define PLANE1_FAIL				0x04
#define PLANE2_FAIL				0x08
#define PLANE3_FAIL				0x10
#define NAND_STATUS_PER 	0x20
#define NAND_STATUS_READY	0x40
#define NAND_STATUS_NP		0x80

/*
 *    Register Status Bit
 */
/* cfg_12 */
#define WP_DISABLE (1<<4) /* disable write protection */
#define OLDATA_EN (1<<2)  /* enable old data structure */
#define WIDTH_8	0
#define WIDTH_16	(1<<3)
#define SW_DIS_ECC (1<<6)
#define PAGE_512 0
#define PAGE_2K 1
#define PAGE_4K 2
#define DIRECT_MAP (1<<5)
/* cfg_1 */
#define DPAHSE_DISABLE 0x80 /* disable data phase */
#define NAND2NFC  0x40  /* direction : nand to controller */
#define NFC2NAND  0x00  /* direction : controller to controller */
#define SING_RW	0x20 /*	enable signal read/ write command */
#define MUL_CMDS 0x10  /*  support cmd+addr+cmd */
#define NFC_TRIGGER 0x01 /* start cmd&addr sequence */
/* cfg_9 */
#define OOB_RW 0x02
/* cfg_a */
#define NFC_CMD_RDY 0x04
#define NFC_BUSY	0x02  /* command and data is being transfer in flash I/O */
#define FLASH_RDY 0x01  /* flash is ready */
/* cfg_b */
#define B2R	0x08 /* status form busy to ready */
/* cfg_15 */
#define USE_SW_ECC 0x04
#define USE_HW_ECC 0
#define ECC1bit 0
#define ECC4bit 1
#define ECC8bit 2
#define ECC12bit 3
#define ECC16bit 4

#define USE_HW_1bit_ECC (ECC1bit<<8|USE_HW_ECC)
#define USE_HW_4bit_ECC (ECC4bit<<8|USE_HW_ECC)
#define USE_HW_8bit_ECC (ECC8bit<<8|USE_HW_ECC)
#define USE_HW_12bit_ECC (ECC12bit<<8|USE_HW_ECC)
#define USE_HW_16bit_ECC (ECC16bit<<8|USE_HW_ECC)

#define ENABLE_BCH_ECC 0xFFFFFFF8
#define DIS_BCH_ECC 0x04

/* cfg_1d */
#define NFC_IDLE	0x01

/* cfg_23 */
#define READ_RESUME 0x100
/* cfg_25 status */
#define ERR_CORRECT 0x100
#define BCH_ERR     0x1
/* cfg_26 status */
#define BANK_DR     0x800
#define BANK_NUM    0x700
#define BCH_ERR_CNT 0x1f

#define FLASH_UNKNOW 0xFFFF

#define ADDR_COLUMN 1
#define ADDR_PAGE 2
#define ADDR_COLUMN_PAGE 3

/*
 * Standard NAND flash commands
 */
#define NAND_CMD_READ0		0
#define NAND_CMD_READ1		1
#define NAND_CMD_READ2		0x50
#define NAND_CMD_PAGEPROG	0x10
#define NAND_CMD_ERASE1		0x60
#define NAND_CMD_STATUS		0x70
#define NAND_CMD_MULTI_STATUS	0x71
#define NAND_CMD_SEQIN		0x80
#define NAND_CMD_READID		0x90
#define NAND_CMD_ERASE2		0xd0
#define NAND_CMD_RESET		0xff

#define	NAND_DUMMY_READ 0x03
#define NAND_DUMMY_PAGEPROG 0x11
#define NAND_CMD_COPYBACK 0x8A
#define NAND_LAG_READ		0x30

/* Status bits */
#define NAND_STATUS_ERR	0x01

#define DMA_SINGNAL 0
#define DMA_INC4 0x10
#define DMA_INC8 0x20


/* function list */
int NFC_WAIT_READY(void);
int NAND_WAIT_READY(void);
int NFC_WAIT_CMD_READY(void);
int NAND_WAIT_IDLE(void);
int set_nfc_ce(int ce);
int wmt_select_nand(unsigned int chip);

int set_nand_div(unsigned int div);
int wmt_calc_clock(WMTNF_INFO_t *info, unsigned int spec_clk, int *T, int *divisor, int *Thold);
int wmt_get_timing(WMTNF_INFO_t *info, unsigned int nand_timing);

int get_nand_info(int ce, WMTNF_INFO_t *info);
void NanD_Address(int numbytes, unsigned int col, unsigned int row);
int nfc_dma_cfg(unsigned int buf, unsigned int len, unsigned int wr);

int nfc_init(int find, int scan);
int nand_erase(unsigned int start);
int WMTLoadImageFormNAND(unsigned int naddr, unsigned int maddr, unsigned int size);
int WMTSaveImageToNAND(unsigned int naddr, unsigned int saddr, unsigned int size);
int nfc_bch_read_page(unsigned int start, unsigned int maddr, WMTNF_INFO_t *info);
int NFC_CHECK_ECC(void);
int nfc_1bit_ecc_correct(unsigned int start, unsigned int maddr, WMTNF_INFO_t *info);
int nfc_1bit_read_page(unsigned int start, unsigned int maddr, WMTNF_INFO_t *info);
int find_bbt(WMTNF_INFO_t *info);
int WMTEraseNAND(unsigned int start, unsigned int nums, unsigned int all);
int tellme_nandinfo(void);
int nand_readID(unsigned char *id);
int get_pattern_small(unsigned int block, unsigned int *tag, unsigned int *version, WMTNF_INFO_t *info);
int get_pattern_large(unsigned int block, unsigned int *tag, unsigned int *version, WMTNF_INFO_t *info);
int WMTEraseNANDALL(unsigned int all);
int check_block_table(unsigned int change, unsigned int scan);
int nfc_bch_ecc_check(unsigned int maddr);
void nfc_bch_ecc_correct(unsigned int bitcnt, unsigned int maddr);
int creat_bbt(WMTNF_INFO_t *info);
int nand_page_program(unsigned int start, unsigned int maddr, WMTNF_INFO_t *info);
void nfc_ecc_set(unsigned int type, WMTNF_INFO_t *nfinfo);
int nand_read_page(unsigned int start, unsigned int maddr, WMTNF_INFO_t *info);
int nand_write_block(unsigned int start, unsigned int naddr, unsigned int size,
	WMTNF_INFO_t *info, unsigned char *oob);
int nand_read_block(unsigned int start,	unsigned int naddr,	unsigned int size,
	WMTNF_INFO_t *info);
int isbbtbadblock(WMTNF_INFO_t *info, unsigned int addr);
int wmt_change_nand(unsigned int *naddr, unsigned int *chip, unsigned int need,	WMTNF_INFO_t *info);
int update_bbt2ram(WMTNF_INFO_t *info, unsigned int addr);
int update_bbt2flash(WMTNF_INFO_t *info);
int tellme_badblock(unsigned int chip);
int tellme_whereistable(unsigned int chip);

/* test function */
//#define POST_TEST_FUNS

#ifdef POST_TEST_FUNS
int set_blockisbad(unsigned int blk);
int set_blockisgood(unsigned int blk);

#define MAX_ECC_BIT 12

int test_nand_rw(int cnt);
int test_nand_ecc(unsigned int cnt);
int ost_init(void);
int read_ost(int *val);
int test_nand_table(unsigned int chip);
#endif
/*
 *	PDMA API
 */
int nand_init_pdma(void);
int nand_free_pdma(void);
int nand_alloc_desc_pool(unsigned long *DescAddr);
int nand_init_short_desc(unsigned long *DescAddr, unsigned int ReqCount, unsigned long *BufferAddr);
int nand_init_long_desc(unsigned long *DescAddr, unsigned int ReqCount, unsigned long *BufferAddr,
unsigned long *BranchAddr, int End);
int nand_config_pdma(unsigned long *DescAddr, unsigned int dir);
int nand_pdma_handler(void);
#endif
